""" Core module with cli """
import click
import json
import sys
import os
import time
import requests
import cipromote.awx as awx
import cipromote.constants as constants
import cipromote.instances as instances
import cipromote.queries as queries
import cipromote.loginInput as loginInput
import cipromote.login as login
import cipromote.labels as labels
import cipromote.label_versions as label_versions
import cipromote.projects as projects
import cipromote.deployment as deployment
import logging
from termcolor import cprint
from pprint import pprint
from requests import Session
from requests.auth import HTTPBasicAuth
from urllib3.exceptions import InsecureRequestWarning

requests.packages.urllib3.disable_warnings(category=InsecureRequestWarning)
session = Session()
session.verify = False
requests.packages.urllib3.disable_warnings(category=InsecureRequestWarning)
session = Session()
session.verify = False
project_name = 'Admin'

@click.group()
def main():
    """
    This is a cli for promoting motioci labels

    Example:

    "$> cipromote get-instances 'https://server.example.com'"


    Example:

    "$> cipromote get-instances-token "New Dev" "New QA" "BillingCenter""

    """


@main.command("check-env", short_help="Check required environment variables")
def check_env():
    """Prints out the current necessary environment variables"""
    motioci_username = os.getenv("MOTIOCI_USERNAME")
    motioci_password = os.getenv("MOTIOCI_PASSWORD")
    motioci_server = os.getenv("MOTIOCI_SERVERURL")
    motioci_token = os.getenv("MOTIOCI_TOKEN")
    awx_host = os.getenv("AWX_HOST")
    awx_token = os.getenv("AWX_TOKEN")
    print(f"Your environment has {motioci_username} for the variable MOTIOCI_USERNAME")
    print(f"Your environment has {motioci_password} for the variable MOTIOCI_PASSWORD")
    print(f"Your environment has {motioci_server} for the variable MOTIOCI_SERVERURL")
    print(f"Your environment has {motioci_token} for the variable MOTIOCI_TOKEN")
    print(f"Your environment has {awx_host} for the variable AWX_HOST")
    print(f"Your environment has {awx_token} for the variable AWX_TOKEN")


@main.command("get-instances", short_help="Get a list of instances")
def get_instances():
    """Gets a list of instances"""
    server_url = os.getenv("MOTIOCI_SERVERURL")
    constants.CI_URL = server_url
    constants.GRAPH_URL = constants.CI_URL + constants.GRAPH_URL
    print('Sending query to the following url')
    print(constants.GRAPH_URL)
    instance_object = instances.get_instances_default()
    pprint(instance_object)
    return instance_object


@main.command("get-instance", short_help="Get an instance by name")
@click.argument("instance-name")
def get_instance(instance_name):
    """ Get a particular instance by name """
    server_url = os.getenv("MOTIOCI_SERVERURL")
    constants.CI_URL = server_url
    constants.GRAPH_URL = constants.CI_URL + constants.GRAPH_URL
    print('Sending query to the following url')
    print(constants.GRAPH_URL)
    instance_object = instances.get_instances_default()
    for instance in instance_object:
        if instance['node']['name'].lower() == instance_name.lower():
            target_instance = instance['node']
            print(f"The target instance is {target_instance['name']}")
            pprint(target_instance)
    return target_instance


@main.command("get-awx-job-template-by-id", short_help="Get AWX Job Template by ID")
@click.argument("awx-job-template-id")
def get_awx_job_template_by_id(awx_job_template_id):
    """ Gets an AWX Job Template by ID """
    awx_host = os.getenv("AWX_HOST")
    awx_token = os.getenv("AWX_TOKEN")
    job_info = awx.get_awx_template_by_id(awx_job_template_id)
    print(job_info)
    return job_info


@main.command("set-awx-job-template-vars", short_help="Set AWX Job Template vars")
@click.argument("awx-job-template-id")
@click.argument("awx-job-template-vars")
def set_awx_job_template_vars(awx_job_template_id, awx_job_template_vars):
    """ Set an AWX Job Template's default extra variables """
    awx_host = os.getenv("AWX_HOST")
    awx_token = os.getenv("AWX_TOKEN")
    job_template = awx.get_awx_template_by_id(awx_job_template_id)
    job_template.extra_vars = awx_job_template_vars
    job_template.put()
    return job_template


@main.command("monitor-instance", short_help="Check an instance by name every X seconds")
@click.argument("instance-name")
@click.argument("seconds")
def monitor_instance(instance_name, seconds):
    """ Check a particular instance by name every X seconds """
    server_url = os.getenv("MOTIOCI_SERVERURL")
    constants.CI_URL = server_url
    constants.GRAPH_URL = constants.CI_URL + constants.GRAPH_URL
    print('Sending query to the following url')
    print(constants.GRAPH_URL)
    while True:
        instance_object = instances.get_instances_default()
        for instance in instance_object:
            if instance['node']['name'].lower() == instance_name.lower():
                target_instance = instance['node']
                print(f"The target instance is {target_instance['name']}")
                pprint(target_instance)
                time.sleep(int(seconds))
    return target_instance


@main.command("monitor-namespace-notify", short_help="Check namespace instance and notify")
@click.argument("instance-name")
@click.argument("namespaces")
@click.argument("seconds")
@click.argument("awx-job-template-id")
def monitor_instance_namespaces(instance_name, namespaces, seconds, awx_job_template_id):
    """ Check a particular instance by name every X seconds

        Example: cipromote monitor-instance-namespaces Dev InfoCenter,BillingCenter 30
        Args:
            instance-name: The name of the node/instance to monitor
            namespaces: A single string or comma separated strings
            seconds: The number of seconds between each check
            awx_job_template_id: The ID of the template to launch on failure

    """
    awx_host = os.getenv("AWX_HOST")
    awx_token = os.getenv("AWX_TOKEN")
    server_url = os.getenv("MOTIOCI_SERVERURL")
    constants.CI_URL = server_url
    constants.GRAPH_URL = constants.CI_URL + constants.GRAPH_URL
    svc_names = namespaces.split(",")
    consecutive_ups = 0
    consecutive_downs = 0
    logging.basicConfig(
            format='%(asctime)s %(levelname)-8s %(message)s',
            level=logging.INFO,
            datefmt='%Y-%m-%d %H:%M:%S')
    while True:
        instance_object = instances.get_instances_default()
        for instance in instance_object:
            if instance['node']['name'].lower() == instance_name.lower():
                target_instance = instance['node']
                up = []
                for ns in target_instance['namespaces']:
                    up.append(ns['name'].lower())
                for svc in svc_names:
                    if svc.lower() in up:
                        consecutive_ups += 1
                        logging.info(f"{svc} is up in {target_instance['name']} for {consecutive_ups} times in a row.")
                        consecutive_downs = 0
                    else:
                        consecutive_downs += 1
                        logging.error(f"{svc} is down in {target_instance['name']} for {consecutive_downs} times in a row.")
                        if consecutive_downs == 10:
                            awx_job_template_vars = {'namespace': svc, 'env': target_instance['name']}
                            json_vars = json.dumps(awx_job_template_vars)
                            job_template = awx.get_awx_template_by_id(awx_job_template_id)
                            job_template.extra_vars = json_vars
                            job_template.put()
                            job_template.launch()
                        consecutive_ups = 0
                time.sleep(int(seconds))
    return target_instance


@main.command("check-instance-namespace", short_help="Check if a particular namespace is present")
@click.argument("instance-name")
@click.argument("namespace-name")
def check_instance_namespace(instance_name, namespace_name):
    """ Check if a particular namespace is present in a particular instance """
    server_url = os.getenv("MOTIOCI_SERVERURL")
    constants.CI_URL = server_url
    constants.GRAPH_URL = constants.CI_URL + constants.GRAPH_URL
    print('Sending query to the following url')
    print(constants.GRAPH_URL)
    instance_object = instances.get_instances_default()
    for instance in instance_object:
        if instance['node']['name'].lower() == instance_name.lower():
            target_instance = instance['node']
            print(f"The target instance is {target_instance['name']}")
            namespaces = target_instance['namespaces']
            namespace_names = []
            for ns in namespaces:
                namespace_names.append(ns['name'].lower())
    try:
        namespace_names
    except NameError:
        print(f"No matching instance for {instance_name} was found.")
        sys.exit(1)
    print(f"Found the following namespaces {namespace_names}")
    if namespace_name.lower() in namespace_names:
        print(f"{namespace_name} was found in the list of connections")
        return namespace_names
    else:
        print(f"No connection found with the name {namespace_name}")
        sys.exit(1)


@main.command("get-labels", short_help="Get a list of available labels")
def get_labels():
    """Gets a lit of available labels"""
    server_url = os.getenv("MOTIOCI_SERVERURL")
    constants.CI_URL = server_url
    constants.GRAPH_URL = constants.CI_URL + constants.GRAPH_URL
    existing_token = os.getenv("MOTIOCI_TOKEN", '""')
    if (os.getenv("MOTIOCI_TOKEN") == "") or (os.getenv("MOTIOCI_TOKEN") == None):
        print(f"You need a valid token defined in your environment.\n"
                f"You should try check-env to check your environment."
                )
        raise SystemExit(1)
    else:
        constants.X_AUTH_TOKEN = os.getenv("MOTIOCI_TOKEN")
    labels_object = labels.get_labels_default()
    if labels_object == 'None':
        print(f"No labels found, maybe your token is expired?")
        raise SystemExit(1)
    return labels_object


@main.command("get-specific-label", short_help="Get a specific label by name")
@click.argument("instance-name")
@click.argument("namespace-name")
@click.argument("label-name")
def get_specific_label(instance_name, namespace_name, label_name):
    """Gets a specific label by name"""
    server_url = os.getenv("MOTIOCI_SERVERURL")
    constants.CI_URL = server_url
    constants.GRAPH_URL = constants.CI_URL + constants.GRAPH_URL
    existing_token = os.getenv("MOTIOCI_TOKEN", '""')
    if (os.getenv("MOTIOCI_TOKEN") == "") or (os.getenv("MOTIOCI_TOKEN") == None):
        print(f"You need a valid token defined in your environment.\n"
                f"You should try check-env to check your environment."
                )
        raise SystemExit(1)
    else:
        constants.X_AUTH_TOKEN = os.getenv("MOTIOCI_TOKEN")
    labels_object = labels.get_label_specific(instance_name, namespace_name, label_name)
    if labels_object == 'None':
        print(f"No labels found, maybe your token is expired?")
        raise SystemExit(1)
    return labels_object


@main.command("get-instances-token", short_help="Authenticate to two instances")
@click.argument("source-instance-name")
@click.argument("target-instance-name")
@click.argument("namespace-name")
def get_instances_token(source_instance_name, target_instance_name, namespace_name):
    """Authenticate against two instances and generates a token"""
    server_url = os.getenv("MOTIOCI_SERVERURL")
    constants.CI_URL = server_url
    constants.LOGIN_URL = constants.CI_URL + constants.LOGIN_URL
    constants.LOGOUT_URL = constants.CI_URL + constants.LOGOUT_URL
    constants.GRAPH_URL = constants.CI_URL + constants.GRAPH_URL
    motioci_username = os.getenv("MOTIOCI_USERNAME")
    motioci_password = os.getenv("MOTIOCI_PASSWORD")
    motioci_server = os.getenv("MOTIOCI_SERVERURL")
    print(
        f'The source instance is "{source_instance_name}". The target instance'
        f' is "{target_instance_name}". The namespace is "{namespace_name}".'
    )
    existing_token = os.getenv("MOTIOCI_TOKEN", '""')
    if (os.getenv("MOTIOCI_TOKEN") == "") or (os.getenv("MOTIOCI_TOKEN") == None):
        credentials = loginInput.get_login_from_user(
            source_instance_name, target_instance_name, namespace_name
        )
        #print(credentials)
        auth_token = login.login_init(credentials)
        if auth_token is None:
            print("Invalid credentials. Login aborted!")
        else:
            #print("Credentials: ", credentials)
            print("Login successful!")
            print("x-auth_token: ", auth_token)
            #os.environ.setdefault('MOTIOCI_TOKEN', auth_token)
            #os.environ['MOTIOCI_TOKEN'] = auth_token
            #print(os.getenv('MOTIOCI_TOKEN'))
            constants.X_AUTH_TOKEN = os.getenv("MOTIOCI_TOKEN")
    else:
        print(
            f"Found a token in environment variable MOTIOCI_TOKEN. {os.getenv('MOTIOCI_TOKEN')}\n"
            f"If you want to get a new token, clear your environment variable."
        )
        constants.X_AUTH_TOKEN = os.getenv("MOTIOCI_TOKEN")


@main.command("promote-label", short_help="Promote a label to a new instance")
@click.argument("source-instance-name")
@click.argument("target-instance-name")
@click.argument("namespace-name")
@click.argument("label-name")
def promote_label(source_instance_name, target_instance_name, namespace_name, label_name):
    """Promote a label from one instance to another"""
    global project_name
    server_url = os.getenv("MOTIOCI_SERVERURL")
    constants.CI_URL = server_url
    constants.LOGIN_URL = constants.CI_URL + constants.LOGIN_URL
    constants.LOGOUT_URL = constants.CI_URL + constants.LOGOUT_URL
    constants.GRAPH_URL = constants.CI_URL + constants.GRAPH_URL
    motioci_username = os.getenv("MOTIOCI_USERNAME")
    motioci_password = os.getenv("MOTIOCI_PASSWORD")
    existing_token = os.getenv("MOTIOCI_TOKEN", '""')
    if (os.getenv("MOTIOCI_TOKEN") == "") or (os.getenv("MOTIOCI_TOKEN") == None):
        print(f"You need a valid token defined in your environment"
                f"You should try check-env to check your environment."
                )
        raise SystemExit(1)
    else:
        print(
            f"Found a token in environment variable MOTIOCI_TOKEN. {os.getenv('MOTIOCI_TOKEN')}\n"
            f"If you want to get a new token, clear your environment variable and run get-instances-token command."
        )
        constants.X_AUTH_TOKEN = os.getenv("MOTIOCI_TOKEN")
    source_instance_id = instances.get_instance_id(source_instance_name)
    print(f"Instance {source_instance_name} has an ID of {source_instance_id}.")
    target_instance_id = instances.get_instance_id(target_instance_name)
    print(f"Instance {target_instance_name} has an ID of {target_instance_id}.")
    project_id = projects.get_project_id(source_instance_id, project_name)
    print(f"The project {project_name} has an ID of {project_id}.")
    label_list = labels.get_label_id(project_id, label_name)
    try:
        label_id = label_list[0]['node']['id']
        print(f"The label {label_name} has an ID of {label_id}.")
    except TypeError:
        print("The label was not found.")
        exit(1)
    except Exception as error:
        print("An error occurred: ", error)
        exit(1)
    try:
        label_version_id = label_versions.get_version_id_default(label_id)
        print(f"The Label ID {label_id} has a version of {label_version_id}.")
    except Exception as error:
        print("Failed to get the version ID of the label.", error)
        exit(1)
    namespace_id = loginInput.get_namespace_id(source_instance_name, namespace_name)
    try:
        print(f"Making the label version promote call")
        #print(f"target_instance_id {target_instance_id}")
        #print(f"label_version_id {label_version_id}")
        #print(f"namespace_id {namespace_id}")
        #print(f"motioci_username {motioci_username}")
        #print(f"motioci_password {motioci_password}")
        source_label_input = {'labelId': label_id}
        #print(f"source_label_input: {source_label_input}")
        #print(f"source_instance_id: {source_instance_id}")
        deployment_target_input = {'instance': {'id': target_instance_id, 'authentication': {'password': {'namespaceId': namespace_id,
                                                                                                          'username': motioci_username,
                                                                                                          'password': motioci_password}},
                                                'labelName': label_name}}
        #print(f"deployment_target_input: {deployment_target_input}")
        #label_versions.promote_label_version_call_standard_auth(target_instance_id, label_version_id, namespace_id, motioci_username, motioci_password)
        deployment.deploy(source_label_input, source_instance_id, deployment_target_input)
    except Exception as error:
        print("Failed to promote the label", error)
    return



'''
Example queries using GraphiQL API
'''

GET_INSTANCES = '''
query instancesQuery{
    instances {
        edges{
            node{
                id
                name
                namespaces{
                    name
                    id
                }
            }
        }
    }
}
'''
'''
Query for retrieving instances that can be logged into.
Used in instances.get_instances_default()
'''

GET_UNVERSIONED_INSTANCES = '''
query instancesQuery{
    unversionedInstances {
        edges{
            node{
                id
                name
                namespaces{
                    name
                    id
                }
            }
        }
    }
}
'''
'''
Query for retrieving instances that can be logged into.
Used in instances.get_instances_default()
'''

GET_INSTANCE_ID = '''
query instanceIdQuery($instanceName: String!){
  instances(filter:{name:{equals:$instanceName}}){
    edges{
      node{
        id
      }
    }
  }
}
'''
'''
Query for retrieving instance id.
Used in instances.get_instance_id(instance_name)
'''
GET_UNVERSIONED_INSTANCE_ID = '''
query instanceIdQuery($instanceName: String!){
  unversionedInstances(filter:{name:{equals:$instanceName}}){
    edges{
      node{
        id
      }
    }
  }
}
'''
'''
Query for retrieving instance id.
Used in instances.get_instance_id(instance_name)
'''

GET_PROJECTS_LS = '''
query projectQuery{
  instances {
    edges{
      node{
        projects{
          edges{
          node{
            id
            name
            description
            }
          }
        }
      }
    }
  }
}
'''
'''
Query for retrieving all available projects.
Used in projects.get_projects_default()
'''

GET_PROJECTS_LS_INSTANCE = '''
query projectQueryInstance($instanceName: String){
  instances(filter:{name:{equals:$instanceName}}){
    edges{
      node{
        projects{
          edges{
            node{
              id
              name
              description
            }
          }
        }
      }
    }
  }
}
'''
'''
Query for retrieving specific projects.
Used in projects.get_projects_specific(instance_name)
'''

GET_PROJECTS_LS_INSTANCE_ID = '''
query projectQueryInstance($instanceId: [Long]){
  instances(filter:{id:{in:$instanceId}}){
    edges{
      node{
        projects{
          edges{
            node{
              id
              name
              description
            }
          }
        }
      }
    }
  }
}
'''
'''
Query for retrieving specific projects.
Used in projects.get_valid_projects(instance_id)
'''

GET_PROJECT_ID = '''
query projectIdQuery($instanceId: [Long], $projectName: String){
  instances(filter:{id:{in:$instanceId}}){
    edges{
      node{
        projects(filter:{name:{equals:$projectName}}){
          edges{
            node{
              id
            }
          }
        }
      }
    }
  }
}
'''
'''
Query for retrieving project id.
Used in projects.get_project_id(source_instance_id, project_name)
'''

GET_LABEL_ID = '''
query labelIdQuery($projectId: Long!, $labelName: String){
  project(id:$projectId){
    labels(filter:{name:{equals:$labelName}}){
      edges{
        node{
          id
        }
      }
    }
  }
}
'''
'''
Query for retrieving label id.
Used in labels.get_label_id(project_id, label_name)
'''

CREATE_LABEL = '''
mutation createLabelMutation($projectId: Long!, $labelName: String!, $versionedItemIds: [Long!]){
  createLabel(input:{projectId:$projectId, name:$labelName, versionedItemIds:$versionedItemIds}){
    name
    description
    id
  }
}
'''
'''
Mutation for creating label.
Used in labels.create_label(project_id, label_name, versioned_item_ids)
'''

DEPLOY_LABEL = '''
mutation deploy($input: DeploymentInput!){
    deploy(input: $input){
        deployment{
          id: id
        }
    }
}
'''



'''
Mutation for promoting a label to a versioned instance.
Used in label_versions.promote_label_version_call(target_instance_id, label_version_id)
'''

GET_VERSION_ID = '''
query versionIdQuery($labelId: Long!){
  label(id:$labelId){
    labelVersions{
     edges{
      node{
        id
        version
      }
     }
    }
  }
}
'''
'''
Query for retrieving latest label version id.
Used in label_versions.get_version_id_default(label_id) and
label_versions.get_version_id_specific(label_id, label_version)
'''

GET_NON_DEL_VERSION = '''
query nonDeletedVersionItems($projectId: Long!, $versionItemPath: String){
    project(id:$projectId){
      versionedItems(currentOnly: true, filter:{path:{starts:$versionItemPath}}) {
        edges {
          node {
            id
          }
        }
      }
    }
}'''
'''
Query for retrieving non-deleted version items.
Used in label_versions.get_version_ids(project_id, search_path)
'''

GET_DEPLOYMENT_STATE = '''
query deploymentState($id: Long!){
  deployment(id: $id){
    state
  }
}
'''
'''
Query for promotion status.
Used in label_versions.promotion_monitoring(promotion_id)
'''

GET_VERSION_ITEM_LS = '''
query getVersionItemLs{
    instances {
        edges {
            node {
                versionedItems {
                    edges {
                        node { 
                        id
                        instanceId
                        prettyPath
                        type
                        version
                        revisionType
                        clientMajor
                        dateTime
                        comment
                        }
                    }
                }
            }
        }
    }
}
'''
'''
Query for retrieving all versioned items.
Used by versioned_items.get_versioned_items_default()
'''

GET_VERSION_ITEM_LS_SPECIFIC = '''
query getVersionItemLsSpecific($instanceName:String, $projectName:String, $path:StringExpression, $currentOption:Boolean){
    instances(filter:{name:{equals:$instanceName}}) {
          edges {
            node {
              projects(filter:{name:{equals:$projectName}}) {
                edges {
                  node {
                    versionedItems(currentOnly:$currentOption, filter:{path:$path}) {
                      edges {
                        node {
                          id
                          prettyPath
                          type
                          version
                          revisionType
                          clientMajor
                          dateTime
                          comment
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
}
'''
'''
Query for retrieving specific versioned items.
Used by versioned_items.get_versioned_items_specific(instance_name, project_name, search_path, current_only)
'''

GET_LABEL_VERSIONS_SPECIFIC = '''
query getLabelVersionSpecific($sourceInstanceName:String, $projectName:String, $labelName: String){
        instances(filter:{name:{equals:$sourceInstanceName}}) {
          edges {
            node {
              projects(filter:{name:{equals:$projectName}}) {
                edges {
                  node {
                    labels(filter:{name:{equals:$labelName}}){
                    edges{
                      node{
                        name
                        labelVersions{
                            edges{
                                node{
                                  version
                                  id
                                  comment
                                }
                            }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
   }
}
'''
'''
Query for retrieving specific label version.
Used in label_versions.get_label_version_specific(source_instance_name, project_name, label_name)
'''

GET_LABEL_VERSIONS = '''
query getLabelVersion{
        instances {
              edges {
                node {
                  projects {
                    edges {
                      node {
                        labels{
                        edges{
                      node{
                        name
                        labelVersions{
                                edges{
                            node{
                              version
                              id
                              comment
                            }
                          }
                        }
                      }
                    }
                   }
                  }
                }
              }
            }
          }
        }
}
'''
'''
Query for retrieving all label version.
Used in label_versions.get_label_version_default()
'''

GET_ALL_LABELS = '''
query getAllLabels{
  instances {
    edges {
      node {
        projects {
          edges {
            node {
              name
                labels {
                      edges {
                        node {
                          id
                        name
                        }
                    }
                }
            }
          }
        }
      }
    }
  }
}
'''
'''
Query for retrieving all labels.
Used in labels.get_labels_default()
'''

GET_LABELS_SPECIFIC = '''
query getLabelsSpecific($instanceName: String!, $projectName: String){
  instances(filter:{name:{equals:$instanceName}}) {
    edges{
      node{
        projects(filter:{name:{equals:$projectName}}){
        edges{
            node{
              labels{
                edges{
                  node{
                    name
                    id
                    description
                    labelType
                  }
                }
              }
            }
          }
        }
      }
    }
  }
}
'''
'''
Query for retrieving specific labels.
Used in labels.get_labels_specific(instance_name, project_name)
'''

GET_LABEL_SPECIFIC = '''
query getLabelSpecific($instanceName: String, $projectName: String, $labelName: String){
  instances(filter:{name:{equals:$instanceName}}) {
    edges{
      node{
        projects(filter:{name:{equals:$projectName}}){
        edges{
          node{
                labels(filter:{name:{equals:$labelName}}){
                  edges{
                    node{
                      name
                      id
                      description
                      labelVersions {
                        edges {
                          node {
                            versionedItems {
                              edges {
                                node {
                                  id
                                  path
                                }
                              }
                            }
                          }
                        }
                      } 
                  }
                }
              }
            }
          }
        }
      }
    }
  }
}
'''
'''
Query for retrieving a specific label.
Used in labels.get_labels_specific(instance_name, project_name, label_name)
'''


GET_VERSIONED_ITEM_BY_ID = '''
query getVersionedItemById($id:Long!){
  versionedItem(id:$id){
    id
  }
}
'''
'''
Query for retrieving a specific versioned item id.
Used in versioned_item.get_versioned_item_by_id(id)
'''

GET_NON_DEL_VERSION_PAGE = '''
query nonDeletedVersionItemsPage($projectId: Long!, $versionItemPath: String, $startCursor: String) {
  project(id: $projectId) {
    versionedItems(
      after: $startCursor,
      currentOnly: true, filter: {path: {starts: $versionItemPath}}, sort: PATH_ASC) {
      pageInfo {
        startCursor
        endCursor
        hasNextPage
      }
      edges {
        cursor
        node {
          id
          path
        }
      }
    }
  }
}
'''

GET_VERSIONED_NAMESPACES = '''
query getVersionedNamespaces($id: Long!){
  instance(id:$id){
    namespaces{
      id
    }
  }
}
'''

GET_UNVERSIONED_NAMESPACES = '''
query getUnversionedNamespaces($id: Long!){
  unversionedInstance(id:$id){
    namespaces{
      id
    }
  }
}
'''


import requests
import constants
import queries
from getpass import getpass

# Available options:
valid_instance_names = {}
valid_namespace_names = {}

# String literals
login_str = "{{"'"namespaceId"'":"'"{namespaceIn}"'","'"username"'":"'"{usernameIn}"'","'"password"'":"'"{passwordIn}"'","'"instanceId"'":"'"{instanceIn}"'"}}"""
comma_str = ", "
starting_str = "["
ending_str = "]"


# Set the instances and namespaces that the user can log in to based on their server link.
def get_instance_info_from_server():
    response = requests.post(constants.GRAPH_URL,
                             json={'query': queries.GET_INSTANCES}, verify=False)
    instance_array = response.json()["data"]["instances"]["edges"]
    for instance in instance_array:
        valid_instance_names.update({instance["node"]["name"]: instance["node"]["id"]})
        for namespace in instance["node"]["namespaces"]:
            valid_namespace_names.update({namespace["name"]: namespace["id"]})
    return


# Ask the user to input necessary info for logging in. Returns credential string to ci-cli.
def get_login_from_user():
    get_instance_info_from_server()
    print("Enter credentials below. Press q to stop.")
    credentials = ""
    firstCred = True
    # Allow user to input credentials only two times.
    for i in range(2):
        instanceInput = input("Enter instanceName: ")
        while instanceInput not in valid_instance_names and instanceInput != "q":
            print("Invalid instance. Available instances: ",
                  ', '.join(str(key) for key, _ in valid_instance_names.items()))
            instanceInput = input("Enter instanceName: ")
        if instanceInput == "q":
            break
        namespaceNameInput = input("Enter namespaceName: ")
        while namespaceNameInput not in valid_namespace_names and namespaceNameInput != "q":
            print("Invalid namespace. Available namespaces: ",
                  ', '.join(str(key) for key, _ in valid_namespace_names.items()))
            namespaceNameInput = input("Enter namespaceName: ")
        if namespaceNameInput == "q":
            break
        usernameInput = input("Enter username: ")
        if usernameInput == "q":
            break
        # nonecho password entry
        passwordInput = getpass()
        if passwordInput == "q":
            break
        if firstCred:
            credentials = starting_str + login_str.format(namespaceIn=valid_namespace_names.get(namespaceNameInput),
                                                          usernameIn=usernameInput,
                                                          passwordIn=passwordInput,
                                                          instanceIn=valid_instance_names.get(instanceInput))
            firstCred = False
        else:
            credentials = credentials + comma_str + login_str.format(
                namespaceIn=valid_namespace_names.get(namespaceNameInput),
                usernameIn=usernameInput,
                passwordIn=passwordInput,
                instanceIn=valid_instance_names.get(instanceInput))
        print("Login instance saved!")

    if credentials != "":
        print("Logging in...")
        credentials = credentials + ending_str
        return credentials
    return



import os
from awxkit import api, config, utils, cli
from awxkit.api import ApiV2, job_templates
from awxkit.api.resources import resources


def auth_awx_get_resources():
    """ Authenticate with token to AWX and get API resources """
    awx_host = os.getenv("AWX_HOST")
    awx_token = os.getenv("AWX_TOKEN")
    config.base_url = awx_host
    awx_api_client = ApiV2()
    awx_api_client.connection.login(token=awx_token)
    awx_api_client.get(resources)
    return awx_api_client


def get_awx_template_by_id(awx_template_id):
    """ Authenticates to AWX and gets a template by ID """
    awx_api_client = auth_awx_get_resources()
    test_template_object_by_id = awx_api_client.job_templates.get(id=awx_template_id).results[0]
    return test_template_object_by_id



import queries
import constants
import requests
import pprint


# Query for all available versioned items and print.
def get_versioned_items_default():
    response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                             json={'query': queries.GET_VERSION_ITEM_LS}, verify=False)
    instance_array = response.json()["data"]["instances"]["edges"]
    for instance in instance_array:
        versioned_item_array = instance["node"]["versionedItems"]
        if versioned_item_array is not None:
            for versioned_item in versioned_item_array["edges"]:
                pprint.pprint(versioned_item["node"])
                print()
    return


# Query for specific non-deleted versioned items and print.
def get_versioned_items_specific(instance_name, project_name, search_path, current_only):
    option, path = get_option_and_path(search_path)
    variables = {
        'instanceName': instance_name,
        'projectName': project_name,
        'path': {option: path},
        'currentOption': current_only
    }
    response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                             json={'query': queries.GET_VERSION_ITEM_LS_SPECIFIC, 'variables': variables}, verify=False)
    versioned_item_array = response.json()["data"]["instances"]["edges"][0]["node"]["projects"]["edges"][0]["node"][
        "versionedItems"]
    for versioned_item in versioned_item_array["edges"]:
        pprint.pprint(versioned_item["node"])
        print()
    return


# Query for specific non-deleted versioned items and print.
def get_option_and_path(search_path):
    first_colon = search_path.find(':')
    return search_path[0:first_colon], search_path[first_colon + 1:]


def get_versioned_item_by_id(id):
    variables = {
        'id': id
    }
    response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                             json={'query': queries.GET_VERSIONED_ITEM_BY_ID, 'variables': variables}, verify=False)
    return response.json()



import requests
import time
import cipromote.constants as constants
import cipromote.queries as queries

# Query for all label versions available
def get_label_version_default():
    response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                             json={'query': queries.GET_LABEL_VERSIONS}, verify=False)
    instance_array = response.json()["data"]["instances"]["edges"]
    for instance in instance_array:
        project_array = instance["node"]["projects"]["edges"]
        for project in project_array:
            label_array = project["node"]["labels"]["edges"]
            for label_version_item in label_array:
                print(label_version_item)
    return


# Query for specific label versions given source_instance_name, project_name, and label_name
def get_label_version_specific(source_instance_name, project_name, label_name):
    variables = {'sourceInstanceName': source_instance_name, 'projectName': project_name, 'labelName': label_name}
    response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                             json={'query': queries.GET_LABEL_VERSIONS_SPECIFIC, 'variables': variables}, verify=False)
    label_version_list = response.json()["data"]["instances"]["edges"][0]["node"]["projects"]["edges"][0]["node"][
        "labels"]["edges"]
    for label_version_item in label_version_list:
        print(label_version_item)
    return


# Query for latest label version id given a label id
def get_version_id_default(label_id):
    variables = {'labelId': label_id}
    response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                             json={'query': queries.GET_VERSION_ID, 'variables': variables}, verify=False)
    node = response.json()["data"]["label"]["labelVersions"]["edges"][-1]["node"]
    print("label version " + str(node["version"]))
    return node["id"]


# Query for specific label version id given a label id and label version
def get_version_id_specific(label_id, label_version):
    variables = {'labelId': label_id}
    response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                             json={'query': queries.GET_VERSION_ID, 'variables': variables}, verify=False)
    node_list = response.json()["data"]["label"]["labelVersions"]["edges"]
    for node in node_list:
        if node["node"]["version"] == label_version:
            return node["node"]["id"]
    return None


# Retrieves versioned item ids when given a search path and outputs the ids found.
def get_version_ids(project_id, search_path):
    startCursor = None
    print("version item ids:", end=" ")
    ans = "["
    for path_item in search_path:
        for cur_path in path_item:
            variables = {'projectId': project_id, 'versionItemPath': cur_path, 'startCursor': startCursor}
            response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                                     json={'query': queries.GET_NON_DEL_VERSION_PAGE, 'variables': variables}, verify=False)
            while True:
                has_more = response.json()["data"]["project"]["versionedItems"]["pageInfo"]["hasNextPage"]
                endCursor = response.json()["data"]["project"]["versionedItems"]["pageInfo"]["endCursor"]
                variables = {'projectId': project_id, 'versionItemPath': cur_path, 'startCursor': endCursor}

                ##get every ID and put it in the label
                ##map collect for getting specific values in json
                items = response.json()["data"]["project"]["versionedItems"]["edges"]
                for item in items:
                    item_id = item["node"]["id"]
                    ans += str(item_id) + ","

                response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                                         json={'query': queries.GET_NON_DEL_VERSION_PAGE, 'variables': variables}, verify=False)

                if has_more == False:
                    break

    if ans[-1] != "[":
        ans = ans[:-1]
    ans += "]"
    print(ans)
    return ans


# Mutation for promoting a label given target_instance_id and label_version_id.
# Calls promotion monitoring to prompt user about promotion progress.
def promote_label_version_call_standard_auth(target_instance_id, label_version_id, namespace_id, username, password):
    variables = {'labelVersionId': label_version_id, 'targetInstanceId': target_instance_id,
                 'namespaceId': namespace_id, 'username': username, 'password': password}
    response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                             json={'query': queries.DEPLOY_LABEL_WITH_STANDARD_AUTH, 'variables': variables}, verify=False, timeout=300)
    print_promotion_response(response.json())


def print_promotion_response(response_json):
    print(response_json)
    if "errors" in response_json:
        print("Error performing the promotion:\n ", response_json["errors"])
    else:
        promotion_id = response_json["data"]["deployLabelVersion"]["id"]
        print("promotionID: " + str(promotion_id))
        promotion_monitoring(promotion_id)
        print("Promoted!")


# Prompts user about promotion progress.
def promotion_monitoring(promotion_id):
    promotion_status = ""
    while promotion_status != "DISALLOWED" and promotion_status != "EXECUTED":
        variables = {'id': promotion_id}
        response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                                 json={'query': queries.GET_PROMOTION_STATE, 'variables': variables}, verify=False)
        promotion_status = str(response.json()["data"]["deploymentResult"]["state"])
        print("promotion result: " + promotion_status)
        time.sleep(6)
    return



import requests
import cipromote.constants as constants
import cipromote.queries as queries

# Query for all available instances.
def get_instances_default():
    response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                             json={'query': queries.GET_INSTANCES}, verify=False)
    instance_array = response.json()["data"]["instances"]["edges"]
    return instance_array


def get_unversioned_instances_default():
    response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                             json={'query': queries.GET_UNVERSIONED_INSTANCES}, verify=False)
    instance_array = response.json()["data"]["unversionedInstances"]["edges"]
    return instance_array


# Print instances name and id only. Used to prompt user about available instances.
def print_instances(instance_array):
    for instance in instance_array:
        print("[Name:", instance["node"]["name"], " ID:", instance["node"]["id"], "]", end=" ")
    return


# Query for specific instance id by instance_name.
def get_instance_id(instance_name):
    variables = {'instanceName': instance_name}
    response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                             json={'query': queries.GET_INSTANCE_ID, 'variables': variables}, verify=False)
    print(response)
    instance_id = response.json()["data"]["instances"]["edges"][0]["node"]["id"]
    return instance_id


# Query for specific instance id by instance_name.
def get_unversioned_instance_id(instance_name):
    variables = {'instanceName': instance_name}
    response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                             json={'query': queries.GET_UNVERSIONED_INSTANCE_ID, 'variables': variables}, verify=False)
    instance_id = response.json()["data"]["unversionedInstances"]["edges"][0]["node"]["id"]
    return instance_id


# Check if instance_id is valid in instance_array
def check_if_valid_instance_id(instance_array, instance_id):
    for instance in instance_array:
        if instance_id == instance["node"]["id"]:
            return True
    return False


# Check if instance_name is valid in instance_array
def check_if_valid_instance_name(instance_array, instance_name):
    for instance in instance_array:
        if instance_name == instance["node"]["name"]:
            return True
    return False



import requests
import constants
import queries


# Query for all available projects.
def get_projects_default():
    response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                             json={'query': queries.GET_PROJECTS_LS})
    instance_array = response.json()["data"]["instances"]["edges"]
    for instance in instance_array:
        project_list = instance["node"]["projects"]["edges"]
        for project_item in project_list:
            print(project_item)
    return


# Query for specific projects given instance_name
def get_projects_specific(instance_name):
    variables = {'instanceName': instance_name}
    response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                             json={'query': queries.GET_PROJECTS_LS_INSTANCE, 'variables': variables})
    project_list = response.json()["data"]["instances"]["edges"][0]["node"]["projects"]["edges"]
    for project_item in project_list:
        print(project_item)
    return


# Retrieve project_id when given source_instance_id and project_name.
def get_project_id(source_instance_id, project_name):
    variables = {'instanceId': source_instance_id, 'projectName': project_name}
    response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                             json={'query': queries.GET_PROJECT_ID, 'variables': variables})
    project_id = response.json()["data"]["instances"]["edges"][0]["node"]["projects"]["edges"][0]["node"]["id"]
    return project_id


# Query for all projects valid given an instance_id. Used for validating user inputted project ids/names
def get_valid_projects(instance_id):
    variables = {'instanceId': instance_id}
    response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                             json={'query': queries.GET_PROJECTS_LS_INSTANCE_ID, 'variables': variables})
    project_list = response.json()["data"]["instances"]["edges"][0]["node"]["projects"]["edges"]
    return project_list


# Print available projects. Used to prompt user about available projects.
def print_projects(project_list):
    for project in project_list:
        print("[Name:", project["node"]["name"], " ID:", project["node"]["id"], "]", end=" ")
    return


# Check if project_id is valid in project_list
def check_if_valid_project_id(project_list, project_id):
    for project in project_list:
        if project_id == project["node"]["id"]:
            return True
    return False


# Check if project_name is valid in project_list
def check_if_valid_project_name(project_list, project_name):
    for project in project_list:
        if project_name == project["node"]["name"]:
            return True
    return False



import constants
import requests


def login_init(login_instance_info):
    response = requests.post(constants.LOGIN_URL, data=login_instance_info, verify=False)
    return response.headers.get("x-auth-token")



import requests
import time
import cipromote.constants as constants
import cipromote.queries as queries


# Mutation for promoting a label given target_instance_id and label_version_id.
# Calls deployment monitoring to prompt user about deployment progress.

def deploy(source_label_input, source_instance_id, deployment_target_input):

    variables = {'input':{'source': {'instanceId': source_instance_id, 'label': source_label_input},
                          'target':deployment_target_input}}

    response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                             json={'query': queries.DEPLOY_LABEL, 'variables': variables}, verify=False)
    print_deployment_response(response.json())

# Calls promotion monitoring to prompt user about promotion progress.
def print_deployment_response(response_json):
    if "errors" in response_json:
        print("Error performing the deployment:\n ", response_json["errors"])
    else:
        deployment_id = response_json["data"]["deploy"]["deployment"]["id"]
        print("deploymentID: " + str(deployment_id))
        deployment_monitoring(deployment_id)
        print("Deployed!")


# Prompts user about promotion progress.
def deployment_monitoring(deployment_id):
    deployment_status = ""
    while deployment_status != "DISALLOWED" and deployment_status != "EXECUTED":
        variables = {'id': deployment_id}
        response = requests.post(constants.GRAPH_URL, headers={'x-auth-token': constants.X_AUTH_TOKEN},
                                 json={'query': queries.GET_DEPLOYMENT_STATE, 'variables': variables}, verify=False)
        deployment_status = str(response.json()["data"]["deployment"]["state"])
        print("deployment result: " + deployment_status)
        time.sleep(6)
    return
